//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Mask/MaskGraphicsScene.cpp
//! @brief     Implements class MaskGraphicsScene
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Mask/MaskGraphicsScene.h"
#include "GUI/Model/Data/Data2DItem.h"
#include "GUI/Model/Mask/MaskContainerItem.h"
#include "GUI/Model/Mask/MaskContainerModel.h"
#include "GUI/Model/Mask/MaskItems.h"
#include "GUI/Model/Project/ProjectDocument.h"
#include "GUI/View/Mask/MaskGraphicsProxy.h"
#include "GUI/View/Mask/MaskViewFactory.h"
#include "GUI/View/PlotUtil/ColorMap.h"
#include "GUI/View/ProShape/SceneAdaptor.h"
#include "GUI/View/Shape/PolygonDisplay.h"
#include <QGraphicsSceneContextMenuEvent>
#include <QItemSelection>
#include <QItemSelectionModel>

namespace {

const qreal min_distance_to_create_rect = 10;

} // namespace


MaskGraphicsScene::MaskGraphicsScene(QObject* parent)
    : QGraphicsScene(parent)
    , m_colorMap(new ColorMap)
{
    setSceneRect(0, 0, 800, 600);
    connect(this, &MaskGraphicsScene::selectionChanged, this,
            &MaskGraphicsScene::onSceneSelectionChanged, Qt::UniqueConnection);
}

MaskGraphicsScene::~MaskGraphicsScene() = default;

void MaskGraphicsScene::associateItems(Data2DItem* data2DItem, MaskContainerItem* maskContainerItem,
                                       QItemSelectionModel* selectionModel)
{
    ASSERT(selectionModel);
    m_selectionModel = selectionModel;
    connect(m_selectionModel, &QItemSelectionModel::selectionChanged, this,
            &MaskGraphicsScene::onSessionSelectionChanged, Qt::UniqueConnection);

    if (!data2DItem)
        return;

    m_data2DItem = data2DItem;

    if (!maskContainerItem)
        return;

    MaskContainerModel* maskContainerModel = maskContainerItem->model();
    ASSERT(maskContainerModel);

    if (maskContainerModel != m_maskContainerModel) {
        disconnectMaskContainer(m_maskContainerModel);
        m_maskContainerItem = maskContainerItem;
        m_maskContainerModel = maskContainerModel;
        connectMaskContainer(m_maskContainerModel);
        resetScene();
        updateScene();
    }
}

void MaskGraphicsScene::dissociateItems()
{
    m_data2DItem = nullptr;
    disconnectMaskContainer(m_maskContainerModel);

    m_maskContainerModel = nullptr;
    resetScene();
}

void MaskGraphicsScene::updateSize(const QSize& newSize)
{
    for (QGraphicsItem* graphicsItem : items()) {
        if (auto* proxy = dynamic_cast<MaskGraphicsProxy*>(graphicsItem)) {
            proxy->resize(newSize);
            setSceneRect(0, 0, newSize.width(), newSize.height());
            proxy->setPos(0, 0);
        }
    }
}

//  ************************************************************************************************
//  public slots
//  ************************************************************************************************

void MaskGraphicsScene::onActivityChanged(MaskEditorFlags::Activity value)
{
    if (!m_proxy)
        return;

    if (m_context.isActivityRequiresDrawingCancel(value))
        cancelCurrentDrawing();

    m_context.setActivityType(value);
    setInPanAndZoomMode(m_context.isInZoomMode());

    updateCursors();
}

void MaskGraphicsScene::onMaskValueChanged(MaskEditorFlags::MaskValue value)
{
    m_context.setMaskValue(value);
}

void MaskGraphicsScene::onRowsInserted(const QModelIndex&, int, int)
{
    updateScene();
}

void MaskGraphicsScene::onRowsAboutToBeRemoved(const QModelIndex&, int first, int last)
{
    m_block_selection = true;
    for (int irow = first; irow <= last; ++irow) {
        if (MaskItem* maskItem = m_maskContainerModel->maskContainer->at(irow))
            removeItemViewFromScene(maskItem);
    }
    m_block_selection = false;
}

void MaskGraphicsScene::cancelCurrentDrawing()
{
    if (isDrawingInProgress()) {
        ASSERT(m_currentItem);
        m_maskContainerModel->removeMask(dynamic_cast<MaskItem*>(m_currentItem));
        setDrawingInProgress(false);
    }
}

void MaskGraphicsScene::resetScene()
{
    m_block_selection = true;

    clearSelection();

    clear();
    m_ItemToView.clear();
    m_proxy = nullptr;
    m_adaptor.reset(new SceneAdaptor);

    m_block_selection = false;
}

//! Main method to update scene on various changes in the model.

void MaskGraphicsScene::updateScene()
{
    if (!m_maskContainerModel)
        return;

    updateProxyWidget();
    updateViews();
    setZValues();
}

//  ************************************************************************************************
//  private slots
//  ************************************************************************************************

//! Propagates selection from model to scene.

void MaskGraphicsScene::onSessionSelectionChanged(const QItemSelection&, const QItemSelection&)
{
    if (m_block_selection)
        return;

    if (!m_selectionModel)
        return;

    m_block_selection = true;

    for (auto it = m_ItemToView.begin(); it != m_ItemToView.end(); ++it) {
        QModelIndex index = m_maskContainerModel->indexOfItem(it.key());
        if (index.isValid())
            it.value()->setSelected(m_selectionModel->isSelected(index));
    }
    m_block_selection = false;
}

//! Propagates selection from scene to model.

void MaskGraphicsScene::onSceneSelectionChanged()
{
    if (m_block_selection)
        return;

    if (!m_selectionModel)
        return;

    m_block_selection = true;

    m_selectionModel->clearSelection();

    for (QGraphicsItem* graphicsItem : selectedItems()) {
        if (auto* view = dynamic_cast<IShapeDisplay*>(graphicsItem)) {
            QModelIndex itemIndex = m_maskContainerModel->indexOfItem(view->parameterizedItem());
            ASSERT(itemIndex.isValid());
            if (!m_selectionModel->isSelected(itemIndex))
                m_selectionModel->select(itemIndex, QItemSelectionModel::Select);
        }
    }
    m_block_selection = false;
}

void MaskGraphicsScene::onMaskMove()
{
    if (m_mouse_is_pressed)
        gProjectDocument.value()->setModified(); // manual mask movement
}

//  ************************************************************************************************
//  private
//  ************************************************************************************************

void MaskGraphicsScene::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
    if (Qt::LeftButton)
        m_mouse_is_pressed = true;

    if (event->buttons() & Qt::RightButton) {
        if (isDrawingInProgress())
            cancelCurrentDrawing();
        else
            makeViewAtMousePosSelected(event);
        return;
    }

    if (isValidForPolygonDrawing(event))
        processPolygonItem(event);
    else if (isValidForLineDrawing(event))
        processLineItem(event);
    else if (isValidForMaskAllDrawing(event))
        processMaskAllItem(event);
    else if (isValidForRectangleShapeDrawing(event))
        processRectangleOrEllipseItem(event);
    else
        QGraphicsScene::mousePressEvent(event);
}

void MaskGraphicsScene::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
{
    if (isDrawingInProgress() && m_context.isRectangleShapeMode()) {
        processRectangleOrEllipseItem(event);
        return;
    }
    QGraphicsScene::mouseMoveEvent(event);

    if ((isDrawingInProgress() && m_context.isPolygonMode()) || m_context.isLineMode()) {
        m_currentMousePosition = event->scenePos();
        invalidate();
    }
}

//! Finalizes item drawing or pass events to other items.

void MaskGraphicsScene::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
{
    m_mouse_is_pressed = false;

    if (isDrawingInProgress()) {
        if (m_context.isRectangleShapeMode()) {
            clearSelection();
            if (m_currentItem) {
                // drawing ended up with item drawn, let's make it selected
                if (IShapeDisplay* view = m_ItemToView[m_currentItem])
                    view->setSelected(true);
            } else {
                // drawing ended without item to be draw (too short mouse move)
                // making item beneath of mouse release position to be selected
                makeViewAtMousePosSelected(event);
            }
            setDrawingInProgress(false);
        }
    } else
        QGraphicsScene::mouseReleaseEvent(event);
}

//! Draws dashed line to the current mouse position in the case of ungoing
//! line or polygon drawing.

void MaskGraphicsScene::drawForeground(QPainter* painter, const QRectF&)
{
    if (m_currentMousePosition == QPointF())
        return;

    if (PolygonDisplay* polygon = currentPolygon()) {
        painter->setPen(QPen(Qt::black, 1, Qt::DashLine));
        painter->drawLine(QLineF(polygon->lastAddedPoint(), m_currentMousePosition));
    } else {
        if (m_context.isLineMode()) {
            const QRectF& plot_scene_rectangle = m_adaptor->viewportRectangle();
            if (!plot_scene_rectangle.contains(m_currentMousePosition))
                return;

            painter->setPen(QPen(Qt::black, 1, Qt::DashLine));
            if (m_context.isVerticalLineMode()) {
                QPointF p1(m_currentMousePosition.x(), plot_scene_rectangle.bottom());
                QPointF p2(m_currentMousePosition.x(), plot_scene_rectangle.top());
                painter->drawLine(QLineF(p1, p2));
            }
            if (m_context.isHorizontalLineMode()) {
                QPointF p1(plot_scene_rectangle.left(), m_currentMousePosition.y());
                QPointF p2(plot_scene_rectangle.right(), m_currentMousePosition.y());
                painter->drawLine(QLineF(p1, p2));
            }
        }
    }
}

//! Creates item context menu if there is IMaskView beneath the mouse right click.

void MaskGraphicsScene::contextMenuEvent(QGraphicsSceneContextMenuEvent* event)
{
    if (isDrawingInProgress())
        return;

    if (dynamic_cast<IShapeDisplay*>(itemAt(event->scenePos(), QTransform())))
        emit itemContextMenuRequest(event->screenPos());
}

//! Updates proxy widget for intensity data item.

void MaskGraphicsScene::updateProxyWidget()
{
    ASSERT(m_data2DItem);
    if (m_proxy)
        return;

    m_proxy = new MaskGraphicsProxy; // TODO: check if m_proxy is released at the end
    m_proxy->setParent(this);
    m_colorMap = new ColorMap; // TODO: potential memory leak
    m_colorMap->itemToMap(m_data2DItem);
    if (m_proxy->widget() != m_colorMap)
        m_proxy->setWidget(m_colorMap);
    m_adaptor->setColorMapPlot(m_colorMap);
    addItem(m_proxy);
}

//! Recursively runs through the model and creates corresponding views.

void MaskGraphicsScene::updateViews()
{
    ASSERT(m_maskContainerItem);

    IShapeDisplay* maskView = addViewForItem(m_maskContainerItem);

    for (MaskItem* maskItem : m_maskContainerItem->maskItems()) {
        if (!maskItem)
            continue;

        IShapeDisplay* itemView = addViewForItem(maskItem);

        if (itemView && maskView) {
            maskView->addView(itemView);

            // Add views for the points of the PolygonItem
            if (auto polygonItem = dynamic_cast<PolygonItem*>(maskItem)) {
                IShapeDisplay* const polygonView = itemView;
                for (PolygonPointItem* pointItem : polygonItem->points()) {
                    IShapeDisplay* pointView = addViewForItem(pointItem);
                    polygonView->addView(pointView);
                }
            }
        }
    }
}

//! Creates a view for given item.

IShapeDisplay* MaskGraphicsScene::addViewForItem(MaskRoot* item)
{
    ASSERT(item);
    subscribeMaskItem(item);

    IShapeDisplay* view = m_ItemToView[item];
    if (!view) {
        view = MaskViewFactory::createMaskView(item);
        view->setViewSceneAdaptor(m_adaptor.get());
        m_ItemToView[item] = view;
        addItem(view);
    }
    return view;
}

//! Removes single view from scene.

void MaskGraphicsScene::removeItemViewFromScene(MaskRoot* item)
{
    for (auto it = m_ItemToView.begin(); it != m_ItemToView.end(); ++it) {
        if (it.key() == item) {
            // at first, delete views for the points of the PolygonItem
            if (auto* polygonItem = dynamic_cast<PolygonItem*>(item))
                for (PolygonPointItem* pointItem : polygonItem->points())
                    removeItemViewFromScene(pointItem);

            IShapeDisplay* view = it.value();
            view->setSelected(false);
            m_ItemToView.erase(it);
            delete view;
            break;
        }
    }
}

void MaskGraphicsScene::connectMaskContainer(MaskContainerModel* maskContainerModel)
{
    if (!maskContainerModel)
        return;

    connect(maskContainerModel, &QAbstractListModel::modelAboutToBeReset, this,
            &MaskGraphicsScene::resetScene);
    connect(maskContainerModel, &QAbstractListModel::rowsInserted, this,
            &MaskGraphicsScene::onRowsInserted);
    connect(maskContainerModel, &QAbstractListModel::rowsAboutToBeRemoved, this,
            &MaskGraphicsScene::onRowsAboutToBeRemoved);
    connect(maskContainerModel, &QAbstractListModel::rowsRemoved, this,
            &MaskGraphicsScene::updateScene);
    connect(maskContainerModel, &QAbstractListModel::modelReset, this,
            &MaskGraphicsScene::updateScene);
    connect(maskContainerModel, &QAbstractListModel::rowsMoved, this,
            &MaskGraphicsScene::updateScene);
}

void MaskGraphicsScene::disconnectMaskContainer(MaskContainerModel* maskContainerModel)
{
    if (!maskContainerModel)
        return;

    disconnect(maskContainerModel, &QAbstractListModel::modelAboutToBeReset, this,
               &MaskGraphicsScene::resetScene);
    disconnect(maskContainerModel, &QAbstractListModel::rowsInserted, this,
               &MaskGraphicsScene::onRowsInserted);
    disconnect(maskContainerModel, &QAbstractListModel::rowsAboutToBeRemoved, this,
               &MaskGraphicsScene::onRowsAboutToBeRemoved);
    disconnect(maskContainerModel, &QAbstractListModel::rowsRemoved, this,
               &MaskGraphicsScene::updateScene);
    disconnect(maskContainerModel, &QAbstractListModel::modelReset, this,
               &MaskGraphicsScene::updateScene);
    disconnect(maskContainerModel, &QAbstractListModel::rowsMoved, this,
               &MaskGraphicsScene::updateScene);
}

//! Returns true if left mouse bottom click was inside ColorMap viewport rectangle.

bool MaskGraphicsScene::isValidMouseClick(QGraphicsSceneMouseEvent* event) const
{
    if (!m_adaptor)
        return false;
    if (!(event->buttons() & Qt::LeftButton))
        return false;
    if (!m_adaptor->viewportRectangle().contains(event->scenePos()))
        return false;
    return true;
}

//! Returns true if mouse click is valid for rectangular/elliptic/ROI shapes.

bool MaskGraphicsScene::isValidForRectangleShapeDrawing(QGraphicsSceneMouseEvent* event) const
{
    if (isDrawingInProgress())
        return false;
    if (!isValidMouseClick(event))
        return false;
    if (!m_context.isRectangleShapeMode())
        return false;
    if (isAreaContains(event, MaskType::SIZEHANDLE))
        return false;
    if (m_context.isROIMode()) {
        // only one ROI is allowed
        for (MaskRoot* item : m_ItemToView.keys())
            if (dynamic_cast<RegionOfInterestItem*>(item))
                return false;
    }
    return true;
}

//! Returns true if mouse click is in context suitable for polygon drawing.

bool MaskGraphicsScene::isValidForPolygonDrawing(QGraphicsSceneMouseEvent* event) const
{
    if (!isValidMouseClick(event))
        return false;
    if (!m_context.isPolygonMode())
        return false;
    if (!isDrawingInProgress()) {
        if (isAreaContains(event, MaskType::POLYGONPOINT))
            return false;
    }
    return true;
}

//! Returns true if mouse click is in context suitable for line drawing.

bool MaskGraphicsScene::isValidForLineDrawing(QGraphicsSceneMouseEvent* event) const
{
    if (!isValidMouseClick(event))
        return false;
    if (isDrawingInProgress())
        return false;
    if (!m_context.isLineMode())
        return false;
    if (QGraphicsItem* graphicsItem = itemAt(event->scenePos(), QTransform())) {
        if (graphicsItem->type() == MaskType::VERTICALLINE
            || graphicsItem->type() == MaskType::HORIZONTALLINE)
            return false;
    }
    return true;
}

//! Returns true if MaskAllItem can be drawn. Only one item of such type is allowed.

bool MaskGraphicsScene::isValidForMaskAllDrawing(QGraphicsSceneMouseEvent* event) const
{
    if (!isValidMouseClick(event))
        return false;
    if (isDrawingInProgress())
        return false;
    if (!m_context.isMaskAllMode())
        return false;

    for (MaskRoot* item : m_ItemToView.keys())
        if (dynamic_cast<MaskAllItem*>(item))
            return false;
    return true;
}

//! Return true if area beneath the mouse contains views of given type.

bool MaskGraphicsScene::isAreaContains(QGraphicsSceneMouseEvent* event, MaskType viewType) const
{
    for (QGraphicsItem* graphicsItem : this->items(event->scenePos()))
        if (graphicsItem->type() == viewType)
            return true;
    return false;
}

bool MaskGraphicsScene::isDrawingInProgress() const
{
    return m_context.isDrawingInProgress();
}

void MaskGraphicsScene::setDrawingInProgress(bool value)
{
    m_context.setDrawingInProgress(value);
    if (value)
        gProjectDocument.value()->setModified(); // manual mask creation
    else
        m_currentItem = nullptr;
}

//! Sets the state of all views in pan&zoom mode.
//! In pan&zoom mode, the selection is removed, all items cannot receive mouse clicks, and all
//! events are propagated down to ColorMap plot.

void MaskGraphicsScene::setInPanAndZoomMode(bool value)
{
    if (value)
        m_selectionModel->clearSelection();

    Qt::MouseButtons acceptedButton = (value ? Qt::NoButton : Qt::LeftButton);
    for (IShapeDisplay* view : m_ItemToView.values())
        view->setAcceptedMouseButtons(acceptedButton);

    m_proxy->setInZoomMode(value);
}

//! Change cursor to stress that hovered item is movable (when not in PanZoom mode)

void MaskGraphicsScene::updateCursors()
{
    for (auto it = m_ItemToView.begin(); it != m_ItemToView.end(); ++it) {
        if (dynamic_cast<VerticalLineItem*>(it.key()))
            it.value()->setCursor(m_context.isInZoomMode() ? Qt::ArrowCursor : Qt::SizeHorCursor);
        else if (dynamic_cast<HorizontalLineItem*>(it.key()))
            it.value()->setCursor(m_context.isInZoomMode() ? Qt::ArrowCursor : Qt::SizeVerCursor);
    }
}

void MaskGraphicsScene::makeViewAtMousePosSelected(QGraphicsSceneMouseEvent* event)
{
    if (QGraphicsItem* graphicsItem = itemAt(event->scenePos(), QTransform()))
        graphicsItem->setSelected(true);
}

//! Processes RectangleItem and EllipseItem drawing
//! If the mouse move distance with left button down is larger than certain threshold,
//! new item will be created. Further, this function will update size and position
//! of current rectangle if mouse keep moving.

void MaskGraphicsScene::processRectangleOrEllipseItem(QGraphicsSceneMouseEvent* event)
{
    if (!isDrawingInProgress())
        setDrawingInProgress(true);

    QPointF click_pos = event->buttonDownScenePos(Qt::LeftButton);
    QPointF mouse_pos = event->scenePos();
    QLineF line(mouse_pos, click_pos);

    if (!m_currentItem && line.length() > min_distance_to_create_rect) {

        MaskItem* newMaskItem;
        if (m_context.isRectangleMode())
            newMaskItem = new RectangleItem;
        else if (m_context.isEllipseMode())
            newMaskItem = new EllipseItem;
        else if (m_context.isROIMode())
            newMaskItem = new RegionOfInterestItem;
        else
            ASSERT_NEVER;

        m_maskContainerModel->insertMask(0, newMaskItem);
        m_currentItem = newMaskItem; // downcast MaskItem* -> MaskRoot*

        if (!m_context.isROIMode())
            dynamic_cast<MaskItem*>(m_currentItem)->setMaskValue(m_context.getMaskValue());

        m_maskContainerItem->updateMaskNames();

    } else if (!m_currentItem)
        return;

    qreal xmin = m_adaptor->fromSceneX(std::min(click_pos.x(), mouse_pos.x()));
    qreal xmax = m_adaptor->fromSceneX(std::max(click_pos.x(), mouse_pos.x()));
    qreal ymin = m_adaptor->fromSceneY(std::min(click_pos.y(), mouse_pos.y()));
    qreal ymax = m_adaptor->fromSceneY(std::max(click_pos.y(), mouse_pos.y()));

    if (auto* rectItem = dynamic_cast<RectangleItem*>(m_currentItem)) {
        // RectangleItem or RegionOfInterestItem
        rectItem->setXLow(xmin);
        rectItem->setYLow(ymax);
        rectItem->setXUp(xmax);
        rectItem->setYUp(ymin);
    } else if (auto* ellItem = dynamic_cast<EllipseItem*>(m_currentItem)) {
        ellItem->setXCenter((xmax + xmin) / 2);
        ellItem->setYCenter((ymax + ymin) / 2);
        ellItem->setXRadius((xmax - xmin) / 2);
        ellItem->setYRadius((ymin - ymax) / 2);
    }

    // produce views for the created shape
    updateViews();
}

void MaskGraphicsScene::processPolygonItem(QGraphicsSceneMouseEvent* event)
{
    ASSERT(m_context.isPolygonMode());

    if (!m_currentItem) {
        setDrawingInProgress(true);
        PolygonItem* new_poly = new PolygonItem;
        m_maskContainerModel->insertMask(0, new_poly);
        new_poly->setMaskValue(m_context.getMaskValue());
        m_currentItem = new_poly;
        m_selectionModel->clearSelection();
        m_selectionModel->select(m_maskContainerModel->indexOfItem(m_currentItem),
                                 QItemSelectionModel::Select);
        m_maskContainerItem->updateMaskNames();
    }
    ASSERT(dynamic_cast<PolygonItem*>(m_currentItem));

    if (PolygonDisplay* polygon = currentPolygon()) {
        if (polygon->closePolygonIfNecessary()) {
            setDrawingInProgress(false);
            m_currentMousePosition = {};
            return;
        }
    }

    const QPointF click_pos = event->buttonDownScenePos(Qt::LeftButton);
    const double x = m_adaptor->fromSceneX(click_pos.x());
    const double y = m_adaptor->fromSceneY(click_pos.y());
    dynamic_cast<PolygonItem*>(m_currentItem)->addPoint(x, y);
    updateScene();
}

void MaskGraphicsScene::processLineItem(QGraphicsSceneMouseEvent* event)
{
    setDrawingInProgress(true);
    QPointF click_pos = event->buttonDownScenePos(Qt::LeftButton);

    if (m_context.isVerticalLineMode())
        processVerticalLineItem(click_pos);

    if (m_context.isHorizontalLineMode())
        processHorizontalLineItem(click_pos);

    m_selectionModel->clearSelection();
    m_selectionModel->select(m_maskContainerModel->indexOfItem(m_currentItem),
                             QItemSelectionModel::Select);
    m_maskContainerItem->updateMaskNames();
    dynamic_cast<MaskItem*>(m_currentItem)->setMaskValue(m_context.getMaskValue());

    emit lineItemProcessed();

    setDrawingInProgress(false);
}

void MaskGraphicsScene::processVerticalLineItem(const QPointF& pos)
{
    VerticalLineItem* item = new VerticalLineItem;
    m_maskContainerModel->insertMask(0, item);
    m_currentItem = item;
    item->setPosX(m_adaptor->fromSceneX(pos.x()));
}

void MaskGraphicsScene::processHorizontalLineItem(const QPointF& pos)
{
    HorizontalLineItem* item = new HorizontalLineItem;
    m_maskContainerModel->insertMask(0, item);
    m_currentItem = dynamic_cast<MaskRoot*>(item);
    item->setPosY(m_adaptor->fromSceneY(pos.y()));
}

// TODO: check MaskAllItem
void MaskGraphicsScene::processMaskAllItem(QGraphicsSceneMouseEvent* event)
{
    Q_UNUSED(event);
    setDrawingInProgress(true);
    MaskAllItem* item = new MaskAllItem;
    m_maskContainerModel->insertMask(0, item);
    m_currentItem = item;
    m_selectionModel->clearSelection();
    setDrawingInProgress(false);
}

void MaskGraphicsScene::subscribeMaskItem(MaskRoot* item)
{
    if (!item)
        return;

    connect(item, &MaskRoot::maskGeometryChanged, this, &MaskGraphicsScene::lineItemMoved,
            Qt::UniqueConnection);
    connect(item, &MaskRoot::maskGeometryChanged, this, &MaskGraphicsScene::onMaskMove,
            Qt::UniqueConnection);
    connect(item, &MaskRoot::maskToBeDestroyed, this, &MaskGraphicsScene::lineItemDeleted,
            Qt::UniqueConnection);
}

//! Update Z-values of all IMaskView to reflect stacking order.

void MaskGraphicsScene::setZValues()
{
    if (!m_maskContainerItem)
        return;

    int z = m_maskContainerItem->maskItems().size();
    for (MaskItem* maskItem : m_maskContainerItem->maskItems()) {
        if (IShapeDisplay* view = m_ItemToView[maskItem])
            view->setZValue(z);
        --z;
    }
}

//! Returns polygon which is currently under the drawing.

PolygonDisplay* MaskGraphicsScene::currentPolygon() const
{
    if (isDrawingInProgress() && m_context.isPolygonMode() && m_currentItem)
        return dynamic_cast<PolygonDisplay*>(m_ItemToView[m_currentItem]);
    return nullptr;
}
