//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Mask/MaskGraphicsScene.h
//! @brief     Defines class MaskGraphicsScene
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_MASK_MASKGRAPHICSSCENE_H
#define BORNAGAIN_GUI_VIEW_MASK_MASKGRAPHICSSCENE_H

#include "GUI/View/Mask/MaskDrawingContext.h"
#include "GUI/View/ProShape/MaskType.h"
#include <QGraphicsScene>
#include <memory>

class ColorMap;
class Data2DItem;
class IShapeDisplay;
class MaskContainerItem;
class MaskContainerModel;
class MaskEditorAction;
class MaskGraphicsProxy;
class MaskItem;
class MaskRoot;
class PolygonDisplay;
class QItemSelection;
class QItemSelectionModel;
class SceneAdaptor;

//! Graphics scene for MaskEditorCanvas to draw masks on top of intensity data widgets.

class MaskGraphicsScene : public QGraphicsScene {
    Q_OBJECT
public:
    MaskGraphicsScene(QObject* parent = nullptr);
    ~MaskGraphicsScene() override;

    void associateItems(Data2DItem* data2DItem, MaskContainerItem* maskContainerItem,
                        QItemSelectionModel* selectionModel);

    void dissociateItems();

    ColorMap* colorMap() { return m_colorMap; }

    void updateSize(const QSize& newSize);

signals:
    void itemContextMenuRequest(const QPoint& point);
    void lineItemProcessed();
    void lineItemMoved(MaskRoot* sender);
    void lineItemDeleted(MaskRoot* sender);

public slots:
    void onActivityChanged(MaskEditorFlags::Activity value);
    void onMaskValueChanged(MaskEditorFlags::MaskValue value);
    void onRowsInserted(const QModelIndex&, int, int);
    void onRowsAboutToBeRemoved(const QModelIndex&, int first, int last);
    void cancelCurrentDrawing();
    void resetScene();
    void updateScene();

private slots:
    void onSessionSelectionChanged(const QItemSelection& /* selected */,
                                   const QItemSelection& /* deselected */);
    void onSceneSelectionChanged();
    void onMaskMove();

private:
    void mousePressEvent(QGraphicsSceneMouseEvent* event) override;
    void mouseMoveEvent(QGraphicsSceneMouseEvent* event) override;
    void mouseReleaseEvent(QGraphicsSceneMouseEvent* event) override;
    void drawForeground(QPainter* painter, const QRectF& rect) override;
    void contextMenuEvent(QGraphicsSceneContextMenuEvent* event) override;

    void updateProxyWidget();
    void updateViews();
    IShapeDisplay* addViewForItem(MaskRoot* item);
    void removeItemViewFromScene(MaskRoot* item);

    void connectMaskContainer(MaskContainerModel* maskContainerModel);
    void disconnectMaskContainer(MaskContainerModel* maskContainerModel);

    bool isValidMouseClick(QGraphicsSceneMouseEvent* event) const;
    bool isValidForRectangleShapeDrawing(QGraphicsSceneMouseEvent* event) const;
    bool isValidForPolygonDrawing(QGraphicsSceneMouseEvent* event) const;
    bool isValidForLineDrawing(QGraphicsSceneMouseEvent* event) const;
    bool isValidForMaskAllDrawing(QGraphicsSceneMouseEvent* event) const;

    bool isAreaContains(QGraphicsSceneMouseEvent* event, MaskType viewType) const;
    bool isDrawingInProgress() const;
    void setDrawingInProgress(bool value);
    void setInPanAndZoomMode(bool value);
    void updateCursors();

    void makeViewAtMousePosSelected(QGraphicsSceneMouseEvent* event);

    void processRectangleOrEllipseItem(QGraphicsSceneMouseEvent* event);
    void processPolygonItem(QGraphicsSceneMouseEvent* event);
    void processLineItem(QGraphicsSceneMouseEvent* event);
    void processVerticalLineItem(const QPointF& pos);
    void processHorizontalLineItem(const QPointF& pos);
    void processMaskAllItem(QGraphicsSceneMouseEvent* event);
    void subscribeMaskItem(MaskRoot* item);

    void setZValues();
    PolygonDisplay* currentPolygon() const;

    ColorMap* m_colorMap;
    MaskContainerModel* m_maskContainerModel = nullptr;
    MaskContainerItem* m_maskContainerItem = nullptr;
    QItemSelectionModel* m_selectionModel = nullptr;
    QMap<MaskRoot*, IShapeDisplay*> m_ItemToView;
    MaskGraphicsProxy* m_proxy = nullptr;
    std::shared_ptr<SceneAdaptor> m_adaptor;
    bool m_block_selection = false;
    bool m_mouse_is_pressed = false;
    Data2DItem* m_data2DItem = nullptr;
    MaskRoot* m_currentItem = nullptr;
    QPointF m_currentMousePosition;
    MaskDrawingContext m_context;
};

#endif // BORNAGAIN_GUI_VIEW_MASK_MASKGRAPHICSSCENE_H
